@@ -16,7 +16,6 @@
  */
 
 
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -42,6 +41,7 @@ extern const char	*__progname;
 
 /* Global for completion */
 static struct cmd_node *root = NULL;
+const char *ctlname = NULL;
 
 static int
 is_lldpctl(const char *name)
@@ -80,7 +80,10 @@ usage()
 static int
 is_privileged()
 {
-	return (!(getuid() != geteuid() || getgid() != getegid()));
+	/* Check we can access the control socket with read/write
+	 * privileges. The `access()` function uses the real UID and real GID,
+	 * therefore we don't have to mangle with our identity. */
+	return (ctlname && access(ctlname, R_OK|W_OK) == 0);
 }
 
 static char*
@@ -192,9 +195,10 @@ _cmd_complete(int all)
 	char **argv = NULL;
 	int argc = 0;
 	int rc = 1;
-	char *line = malloc(strlen(rl_line_buffer) + 2);
+	size_t len = strlen(rl_line_buffer);
+	char *line = malloc(len + 2);
 	if (!line) return -1;
-	strcpy(line, rl_line_buffer);
+	strlcpy(line, rl_line_buffer, len + 2);
 	line[rl_point]   = 2;	/* empty character, will force a word */
 	line[rl_point+1] = 0;
 
@@ -212,6 +216,7 @@ _cmd_complete(int all)
 		goto end;
 	}
 	/* No completion or several completion available. */
+	free(compl);
 	fprintf(stderr, "\n");
 	rl_forced_update_display();
 	rc = 0;
@@ -234,10 +239,10 @@ cmd_help(int count, int ch)
 }
 #else
 static char*
-readline()
+readline(const char *p)
 {
 	static char line[2048];
-	fprintf(stderr, "%s", prompt());
+	fprintf(stderr, "%s", p);
 	fflush(stderr);
 	if (fgets(line, sizeof(line) - 2, stdin) == NULL)
 		return NULL;
@@ -305,10 +310,12 @@ parse_and_exec(lldpctl_conn_t *conn, const char *fmt, const char *line)
 		log_warnx("lldpctl", "unmatched quotes");
 		return -1;
 	}
-	if (cargc == 0) return 0;
-	n = cmd_exec(conn, fmt, cargc, (const char **)cargv);
+	if (cargc != 0)
+		n = cmd_exec(conn, fmt, cargc, (const char **)cargv);
 	tokenize_free(cargc, cargv);
-	return (n == 0)?-1:1;
+	return (cargc == 0)?0:
+	    (n == 0)?-1:
+	    1;
 }
 
 static struct cmd_node*
@@ -413,11 +420,12 @@ main(int argc, char *argv[])
 	lldpctl_conn_t *conn = NULL;
 	const char *options = is_lldpctl(argv[0])?"hdvf:":"hdsvf:c:u:";
 
-	const char *ctlname = lldpctl_get_default_transport();
 	int gotinputs = 0;
 	struct inputs inputs;
 	TAILQ_INIT(&inputs);
 
+	ctlname = lldpctl_get_default_transport();
+
 	/* Initialize logging */
 	while ((ch = getopt(argc, argv, options)) != -1) {
 		switch (ch) {
@@ -465,6 +473,8 @@ main(int argc, char *argv[])
 
 	/* Process file inputs */
 	while (gotinputs && !TAILQ_EMPTY(&inputs)) {
+		/* coverity[use_after_free]
+		   TAILQ_REMOVE does the right thing */
 		struct input *first = TAILQ_FIRST(&inputs);
 		log_debug("lldpctl", "process: %s", first->name);
 		FILE *file = fopen(first->name, "r");
@@ -525,7 +535,10 @@ main(int argc, char *argv[])
 		goto end;
 	}
 
-	if (gotinputs) goto end;
+	if (gotinputs) {
+		rc = EXIT_SUCCESS;
+		goto end;
+	}
 
 	/* Interactive session */
 #ifdef HAVE_LIBREADLINE
@@ -546,6 +559,8 @@ main(int argc, char *argv[])
 
 end:
 	while (!TAILQ_EMPTY(&inputs)) {
+		/* coverity[use_after_free]
+		   TAILQ_REMOVE does the right thing */
 		struct input *first = TAILQ_FIRST(&inputs);
 		TAILQ_REMOVE(&inputs, first, next);
 		free(first->name);
