@@ -25,6 +25,7 @@
 #include <unistd.h>
 #include <errno.h>
 
+#ifdef ENABLE_PRIVSEP
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
@@ -147,7 +148,7 @@ agent_priv_unix_transport(const char *string, int len, int local)
 	t->flags = NETSNMP_TRANSPORT_FLAG_STREAM;
 
 	if ((t->remote = (u_char *)
-		malloc(strlen(addr.sun_path))) == NULL) {
+		calloc(1, strlen(addr.sun_path) + 1)) == NULL) {
 		agent_priv_unix_close(t);
 		netsnmp_transport_free(t);
 		return NULL;
@@ -204,3 +205,4 @@ agent_priv_register_domain()
 	unixDomain.f_create_from_ostring = agent_priv_unix_create_ostring;
 	netsnmp_tdomain_register(&unixDomain);
 }
+#endif
