@@ -67,7 +67,7 @@ cdp_send(struct lldpd *global,
 	/* Ethernet header */
 	if (!(
 	      POKE_BYTES(mcastaddr, sizeof(mcastaddr)) &&
-	      POKE_BYTES(&hardware->h_lladdr, sizeof(hardware->h_lladdr)) &&
+	      POKE_BYTES(&hardware->h_lladdr, ETHER_ADDR_LEN) &&
 	      POKE_SAVE(pos_len_eh) && /* We compute the len later */
 	      POKE_UINT16(0)))
 		goto toobig;
@@ -94,7 +94,9 @@ cdp_send(struct lldpd *global,
 	/* Chassis ID */
 	if (!(
 	      POKE_START_CDP_TLV(CDP_TLV_CHASSIS) &&
-	      POKE_BYTES(chassis->c_name, strlen(chassis->c_name)) &&
+	      (chassis->c_name?
+		  POKE_BYTES(chassis->c_name, strlen(chassis->c_name)):
+		  POKE_BYTES("", 0)) &&
 	      POKE_END_CDP_TLV))
 		goto toobig;
 
@@ -134,8 +136,10 @@ cdp_send(struct lldpd *global,
 	/* Port ID */
 	if (!(
 	      POKE_START_CDP_TLV(CDP_TLV_PORT) &&
-	      POKE_BYTES(hardware->h_lport.p_descr,
-			 strlen(hardware->h_lport.p_descr)) &&
+	      (hardware->h_lport.p_descr?
+		  POKE_BYTES(hardware->h_lport.p_descr,
+		      strlen(hardware->h_lport.p_descr)):
+		  POKE_BYTES("", 0)) &&
 	      POKE_END_CDP_TLV))
 		goto toobig;
 
@@ -185,7 +189,9 @@ cdp_send(struct lldpd *global,
 	/* Software version */
 	if (!(
 	      POKE_START_CDP_TLV(CDP_TLV_SOFTWARE) &&
-	      POKE_BYTES(chassis->c_descr, strlen(chassis->c_descr)) &&
+	      (chassis->c_descr?
+		  POKE_BYTES(chassis->c_descr, strlen(chassis->c_descr)):
+		  POKE_BYTES("", 0)) &&
 	      POKE_END_CDP_TLV))
 		goto toobig;
 
