@@ -75,6 +75,13 @@ client_handle_set_configuration(struct lldpd *cfg, enum hmsg_type *type,
 		log_debug("rpc", "client asked for immediate retransmission");
 		levent_send_now(cfg);
 	}
+	if (config->c_lldp_portid_type > LLDP_PORTID_SUBTYPE_UNKNOWN &&
+            config->c_lldp_portid_type <= LLDP_PORTID_SUBTYPE_MAX) {
+            log_debug("rpc", "change lldp portid tlv subtype to %d",
+                      config->c_lldp_portid_type);
+            cfg->g_config.c_lldp_portid_type = config->c_lldp_portid_type;
+	    levent_update_now(cfg);
+	}
 	/* Pause/resume */
 	if (config->c_paused != cfg->g_config.c_paused) {
 		log_debug("rpc", "client asked to %s lldpd",
@@ -100,6 +107,12 @@ client_handle_set_configuration(struct lldpd *cfg, enum hmsg_type *type,
 		cfg->g_config.c_iface_pattern = strdup(config->c_iface_pattern);
 		levent_update_now(cfg);
 	}
+	if (config->c_mgmt_pattern) {
+		log_debug("rpc", "change management pattern to %s", config->c_mgmt_pattern);
+		free(cfg->g_config.c_mgmt_pattern);
+		cfg->g_config.c_mgmt_pattern = strdup(config->c_mgmt_pattern);
+		levent_update_now(cfg);
+	}
 	if (config->c_description) {
 		log_debug("rpc", "change chassis description to %s", config->c_description);
 		free(cfg->g_config.c_description);
@@ -112,6 +125,12 @@ client_handle_set_configuration(struct lldpd *cfg, enum hmsg_type *type,
 		cfg->g_config.c_platform = strdup(config->c_platform);
 		levent_update_now(cfg);
 	}
+	if (config->c_hostname) {
+		log_debug("rpc", "change system name to %s", config->c_hostname);
+		free(cfg->g_config.c_hostname);
+		cfg->g_config.c_hostname = strdup(config->c_hostname);
+		levent_update_now(cfg);
+	}
 	if (config->c_set_ifdescr != cfg->g_config.c_set_ifdescr) {
 		log_debug("rpc", "%s setting of interface description based on discovered neighbors",
 		    config->c_set_ifdescr?"enable":"disable");
@@ -214,8 +233,8 @@ client_handle_get_interface(struct lldpd *cfg, enum hmsg_type *type,
 			return output_len;
 		}
 
-	free(name);
 	log_warnx("rpc", "no interface %s found", name);
+	free(name);
 	*type = NONE;
 	return 0;
 }
