@@ -373,6 +373,8 @@ levent_ctl_accept(evutil_socket_t fd, short what, void *arg)
 	    client);
 	bufferevent_enable(client->bev, EV_READ | EV_WRITE);
 	log_debug("event", "new client accepted");
+	/* coverity[leaked_handle]
+	   s has been saved by bufferevent_socket_new */
 	return;
 accept_failed:
 	levent_ctl_free_client(client);
@@ -690,8 +692,14 @@ levent_schedule_cleanup(struct lldpd *cfg)
 	struct lldpd_port *port;
 	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
 		TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
+			if (now >= port->p_lastupdate + port->p_chassis->c_ttl) {
+				tv.tv_sec = 0;
+				log_debug("event", "immediate cleanup on port %s (%ld, %d, %ld)",
+				    hardware->h_ifname, now, port->p_chassis->c_ttl, port->p_lastupdate);
+				break;
+			}
 			next = port->p_chassis->c_ttl - (now - port->p_lastupdate);
-			if (next >= 0 && next < tv.tv_sec)
+			if (next < tv.tv_sec)
 				tv.tv_sec = next;
 		}
 	}
