@@ -20,6 +20,7 @@
 #include <unistd.h>
 #include <ifaddrs.h>
 #include <errno.h>
+#include <ctype.h>
 #include <sys/param.h>
 #include <sys/sysctl.h>
 #include <sys/ioctl.h>
@@ -330,12 +331,38 @@ ifbsd_check_physical(struct lldpd *cfg,
 	iface->type |= IFACE_PHYSICAL_T;
 }
 
+/* Blacklist any dangerous interface. Currently, only p2p0 is blacklisted as it
+ * triggers some AirDrop functionality when we send something on it.
+ *  See: https://github.com/vincentbernat/lldpd/issues/61
+ */
+static void
+ifbsd_blacklist(struct lldpd *cfg,
+    struct interfaces_device_list *interfaces)
+{
+#ifdef HOST_OS_OSX
+	struct interfaces_device *iface = NULL;
+	TAILQ_FOREACH(iface, interfaces, next) {
+		int i;
+		if (strncmp(iface->name, "p2p", 3)) continue;
+		if (strlen(iface->name) < 4) continue;
+		for (i = 3;
+		     iface->name[i] != '\0' && isdigit(iface->name[i]);
+		     i++);
+		if (iface->name[i] == '\0') {
+			log_debug("interfaces", "skip %s: AirDrop interface",
+			    iface->name);
+			iface->flags = 0;
+		}
+	}
+#endif
+}
+
 static struct interfaces_device*
 ifbsd_extract_device(struct lldpd *cfg,
     struct ifaddrs *ifaddr)
 {
 	struct interfaces_device *iface = NULL;
-	struct sockaddr_dl *saddrdl = (struct sockaddr_dl*)ifaddr->ifa_addr;
+	struct sockaddr_dl *saddrdl = ALIGNED_CAST(struct sockaddr_dl*, ifaddr->ifa_addr);
 	if ((saddrdl->sdl_type != IFT_BRIDGE) &&
 	    (saddrdl->sdl_type != IFT_L2VLAN) &&
 	    (saddrdl->sdl_type != IFT_ETHER)) {
@@ -359,7 +386,7 @@ ifbsd_extract_device(struct lldpd *cfg,
 		memcpy(iface->address, LLADDR(saddrdl), ETHER_ADDR_LEN);
 
 	/* Grab description */
-#ifdef IFDESCRSIZE
+#ifdef SIOCGIFDESCR
 #if defined HOST_OS_FREEBSD || defined HOST_OS_OPENBSD
 	iface->alias = malloc(IFDESCRSIZE);
 	if (iface->alias) {
@@ -380,7 +407,7 @@ ifbsd_extract_device(struct lldpd *cfg,
 		}
 	}
 #endif
-#endif /* IFDESCRSIZE */
+#endif /* SIOCGIFDESCR */
 
 	if (ifbsd_check_wireless(cfg, ifaddr, iface) == -1) {
 		interfaces_free_device(iface);
@@ -616,6 +643,7 @@ interfaces_update(struct lldpd *cfg)
 		ifbsd_check_physical(cfg, interfaces, iface);
 	}
 
+	ifbsd_blacklist(cfg, interfaces);
 	interfaces_helper_whitelist(cfg, interfaces);
 	interfaces_helper_physical(cfg, interfaces,
 	    &bpf_ops, ifbpf_phys_init);
@@ -655,6 +683,4 @@ end:
 	interfaces_free_devices(interfaces);
 	interfaces_free_addresses(addresses);
 	if (ifaddrs) freeifaddrs(ifaddrs);
-	return;
-
 }
