@@ -122,7 +122,7 @@ old_iflinux_is_bridge(struct lldpd *cfg,
 		.ifr_data = (char*)&args2
 	};
 
-	strncpy(ifr.ifr_name, iface->name, IFNAMSIZ);
+	strlcpy(ifr.ifr_name, iface->name, IFNAMSIZ);
 
 	if (ioctl(cfg->g_sock, SIOCDEVPRIVATE, &ifr) < 0)
 		/* This can happen with a 64bit kernel and 32bit
@@ -272,8 +272,8 @@ iflinux_get_permanent_mac(struct lldpd *cfg,
 	struct interfaces_device *master;
 	int f, state = 0;
 	FILE *netbond;
-	const char const *slaveif = "Slave Interface: ";
-	const char const *hwaddr = "Permanent HW addr: ";
+	const char *slaveif = "Slave Interface: ";
+	const char *hwaddr = "Permanent HW addr: ";
 	u_int8_t mac[ETHER_ADDR_LEN];
 	char path[SYSFS_PATH_MAX];
 	char line[100];
@@ -551,7 +551,7 @@ iflinux_handle_bond(struct lldpd *cfg, struct interfaces_device_list *interfaces
 				    iface->name);
 				continue;
 			}
-			hardware->h_data = bmaster = calloc(1, sizeof(struct bond_master));
+			hardware->h_data = calloc(1, sizeof(struct bond_master));
 			if (!hardware->h_data) {
 				log_warn("interfaces", "not enough memory");
 				lldpd_hardware_cleanup(cfg, hardware);
@@ -569,9 +569,9 @@ iflinux_handle_bond(struct lldpd *cfg, struct interfaces_device_list *interfaces
 		} else {
 			if (hardware->h_flags) continue; /* Already seen this time */
 			bmaster = hardware->h_data;
-			memset(hardware->h_data, 0, sizeof(struct bond_master));
+			memset(bmaster, 0, sizeof(struct bond_master));
 			bmaster->index = master->index;
-			strncpy(bmaster->name, master->name, IFNAMSIZ);
+			strlcpy(bmaster->name, master->name, IFNAMSIZ);
 			lldpd_port_cleanup(&hardware->h_lport, 0);
 		}
 
@@ -582,7 +582,7 @@ iflinux_handle_bond(struct lldpd *cfg, struct interfaces_device_list *interfaces
 		memcpy(&hardware->h_lladdr, iface->address, ETHER_ADDR_LEN);
 
 		/* Fill information about port */
-		interfaces_helper_port_name_desc(hardware, iface);
+		interfaces_helper_port_name_desc(cfg, hardware, iface);
 
 		/* Fill additional info */
 #ifdef ENABLE_DOT3
@@ -807,11 +807,11 @@ interfaces_update(struct lldpd *cfg)
 		}
 		if (levent_iface_subscribe(cfg, s) == -1)
 			close(s);
+		/* coverity[leaked_handle]
+		   s has been saved by levent_iface_subscribe */
 	}
 
 end:
 	interfaces_free_devices(interfaces);
 	interfaces_free_addresses(addresses);
-	return;
-
 }
