@@ -22,7 +22,6 @@
 #include <unistd.h>
 #include <errno.h>
 #include <assert.h>
-#include <fnmatch.h>
 #include <arpa/inet.h>
 
 /* Generic ethernet interface initialization */
@@ -157,34 +156,6 @@ interfaces_indextointerface(struct interfaces_device_list *interfaces,
 	return NULL;
 }
 
-static int
-pattern_match(char *iface, char *list, int found)
-{
-	char *interfaces = NULL;
-	char *pattern;
-
-	if ((interfaces = strdup(list)) == NULL) {
-		log_warnx("interfaces", "unable to allocate memory");
-		return 0;
-	}
-
-	for (pattern = strtok(interfaces, ",");
-	     pattern != NULL;
-	     pattern = strtok(NULL, ",")) {
-		if ((pattern[0] == '!') &&
-		    ((fnmatch(pattern + 1, iface, 0) == 0))) {
-			/* Blacklisted. No need to search further. */
-			found = 0;
-			break;
-		}
-		if (fnmatch(pattern, iface, 0) == 0)
-			found = 1;
-	}
-
-	free(interfaces);
-	return found;
-}
-
 void
 interfaces_helper_whitelist(struct lldpd *cfg,
     struct interfaces_device_list *interfaces)
@@ -195,11 +166,17 @@ interfaces_helper_whitelist(struct lldpd *cfg,
 		return;
 
 	TAILQ_FOREACH(iface, interfaces, next) {
-		if (iface->flags == 0) continue; /* Already handled by someone else */
-		if (!pattern_match(iface->name, cfg->g_config.c_iface_pattern, 0)) {
-			/* This interface was not found. We flag it. */
+		int m = pattern_match(iface->name, cfg->g_config.c_iface_pattern, 0);
+		switch (m) {
+		case 0:
 			log_debug("interfaces", "blacklist %s", iface->name);
 			iface->flags = 0;
+			continue;
+		case 2:
+			log_debug("interfaces", "whitelist %s (consider it as a physical interface)",
+			    iface->name);
+			iface->type |= IFACE_PHYSICAL_T;
+			continue;
 		}
 	}
 }
@@ -321,12 +298,15 @@ interfaces_helper_chassis(struct lldpd *cfg,
 	struct lldpd_hardware *hardware;
 	char *name = NULL;
 
+	LOCAL_CHASSIS(cfg)->c_cap_enabled &= ~(LLDP_CAP_BRIDGE | LLDP_CAP_WLAN);
 	TAILQ_FOREACH(iface, interfaces, next) {
 		if (iface->type & IFACE_BRIDGE_T)
 			LOCAL_CHASSIS(cfg)->c_cap_enabled |= LLDP_CAP_BRIDGE;
 		if (iface->type & IFACE_WIRELESS_T)
 			LOCAL_CHASSIS(cfg)->c_cap_enabled |= LLDP_CAP_WLAN;
 	}
+	if (LOCAL_CHASSIS(cfg)->c_cap_enabled == 0)
+		LOCAL_CHASSIS(cfg)->c_cap_enabled = LLDP_CAP_STATION;
 
 	if (LOCAL_CHASSIS(cfg)->c_id != NULL &&
 	    LOCAL_CHASSIS(cfg)->c_id_subtype == LLDP_CHASSISID_SUBTYPE_LLADDR)
@@ -344,15 +324,15 @@ interfaces_helper_chassis(struct lldpd *cfg,
 			/* That's odd. Let's skip. */
 			continue;
 
-		name = malloc(sizeof(hardware->h_lladdr));
+		name = malloc(ETHER_ADDR_LEN);
 		if (!name) {
 			log_warn("interfaces", "not enough memory for chassis ID");
 			return;
 		}
 		free(LOCAL_CHASSIS(cfg)->c_id);
-		memcpy(name, hardware->h_lladdr, sizeof(hardware->h_lladdr));
+		memcpy(name, hardware->h_lladdr, ETHER_ADDR_LEN);
 		LOCAL_CHASSIS(cfg)->c_id = name;
-		LOCAL_CHASSIS(cfg)->c_id_len = sizeof(hardware->h_lladdr);
+		LOCAL_CHASSIS(cfg)->c_id_len = ETHER_ADDR_LEN;
 		LOCAL_CHASSIS(cfg)->c_id_subtype = LLDP_CHASSISID_SUBTYPE_LLADDR;
 		return;
 	}
@@ -455,45 +435,60 @@ interfaces_helper_mgmt(struct lldpd *cfg,
 
 /* Fill up port name and description */
 void
-interfaces_helper_port_name_desc(struct lldpd_hardware *hardware,
+interfaces_helper_port_name_desc(struct lldpd *cfg,
+    struct lldpd_hardware *hardware,
     struct interfaces_device *iface)
 {
 	struct lldpd_port *port = &hardware->h_lport;
 
-	/* There are two cases:
-
-	     1. We have a kernel recent enough to support ifAlias
-	     _and_ a non empty ifAlias, then we will use it for
-	     description and use ifname for port ID.
-
-	     2. Otherwise, we will use the MAC address as ID and the
-	     port name in description.
+	/* We need to set the portid to what the client configured.
+	   This can be done from the CLI.
 	*/
-
-	if (iface->alias == NULL || strlen(iface->alias) == 0) {
-		/* Case 2: MAC address and port name */
-		log_debug("interfaces", "use ifname and MAC address for %s",
-		    hardware->h_ifname);
-		port->p_id_subtype = LLDP_PORTID_SUBTYPE_LLADDR;
-		if ((port->p_id =
-			calloc(1, sizeof(hardware->h_lladdr))) == NULL)
+	switch (cfg->g_config.c_lldp_portid_type) {
+	case LLDP_PORTID_SUBTYPE_IFNAME:
+		log_debug("interfaces", "use ifname for %s",
+			  hardware->h_ifname);
+		port->p_id_subtype = LLDP_PORTID_SUBTYPE_IFNAME;
+		port->p_id_len = strlen(hardware->h_ifname);
+		if (port->p_id != NULL) {
+			free(port->p_id);
+		}
+		if ((port->p_id = calloc(1, port->p_id_len)) == NULL)
 			fatal("interfaces", NULL);
-		memcpy(port->p_id, hardware->h_lladdr,
-		    sizeof(hardware->h_lladdr));
-		port->p_id_len = sizeof(hardware->h_lladdr);
+		memcpy(port->p_id, hardware->h_ifname, port->p_id_len);
+		break;
+	case LLDP_PORTID_SUBTYPE_LLADDR:
+		/* fall through so we use the mac address */
+	default:
+		log_debug("interfaces", "use MAC address for %s",
+			  hardware->h_ifname);
+		port->p_id_subtype = LLDP_PORTID_SUBTYPE_LLADDR;
+		if (port->p_id != NULL) {
+			free(port->p_id);
+		}
+		if ((port->p_id = calloc(1, ETHER_ADDR_LEN)) == NULL)
+			fatal("interfaces", NULL);
+		memcpy(port->p_id, hardware->h_lladdr, ETHER_ADDR_LEN);
+		port->p_id_len = ETHER_ADDR_LEN;
+	}
+
+	if (iface->alias != NULL && strlen(iface->alias) != 0) {
+		/* use the actual alias in the port description */
+		log_debug("interfaces", "using alias in description for %s",
+			  hardware->h_ifname);
+		if (port->p_descr != NULL) {
+			free(port->p_descr);
+		}
+		port->p_descr = strdup(iface->alias);
+	} else {
+		/* use the ifname in the port description until alias is set */
+		log_debug("interfaces", "using ifname in description for %s",
+			  hardware->h_ifname);
+		if (port->p_descr != NULL) {
+			free(port->p_descr);
+		}
 		port->p_descr = strdup(hardware->h_ifname);
-		return;
 	}
-	/* Case 1: port name and port description */
-	log_debug("interfaces", "use ifname and ifalias for %s",
-	    hardware->h_ifname);
-	port->p_id_subtype = LLDP_PORTID_SUBTYPE_IFNAME;
-	port->p_id_len = strlen(hardware->h_ifname);
-	if ((port->p_id =
-		calloc(1, port->p_id_len)) == NULL)
-		fatal("interfaces", NULL);
-	memcpy(port->p_id, hardware->h_ifname, port->p_id_len);
-	port->p_descr = strdup(iface->alias);
 }
 
 void
@@ -556,7 +551,7 @@ interfaces_helper_physical(struct lldpd *cfg,
 		memcpy(&hardware->h_lladdr, iface->address, ETHER_ADDR_LEN);
 
 		/* Fill information about port */
-		interfaces_helper_port_name_desc(hardware, iface);
+		interfaces_helper_port_name_desc(cfg, hardware, iface);
 
 		/* Fill additional info */
 		hardware->h_mtu = iface->mtu ? iface->mtu : 1500;
@@ -594,13 +589,11 @@ interfaces_send_helper(struct lldpd *cfg,
 
 		switch (cfg->g_config.c_bond_slave_src_mac_type) {
 		case LLDP_BOND_SLAVE_SRC_MAC_TYPE_LOCALLY_ADMINISTERED:
-			if (*src_mac & MAC_UL_ADMINISTERED_BIT_MASK) {
-				/* If locally administered bit already set,
-				 * use zero mac
-				 */
-				memset(src_mac, 0, ETHER_ADDR_LEN);
+			if (!(*src_mac & MAC_UL_ADMINISTERED_BIT_MASK)) {
+				*src_mac |= MAC_UL_ADMINISTERED_BIT_MASK;
 				break;
 			}
+			/* Fallback to fixed value */
 		case LLDP_BOND_SLAVE_SRC_MAC_TYPE_FIXED:
 			memcpy(src_mac, arbitrary, ETHER_ADDR_LEN);
 			break;
