@@ -94,7 +94,7 @@ lldp_send(struct lldpd *global,
 	      /* LLDP multicast address */
 	      POKE_BYTES(mcastaddr, sizeof(mcastaddr)) &&
 	      /* Source MAC address */
-	      POKE_BYTES(&hardware->h_lladdr, sizeof(hardware->h_lladdr)) &&
+	      POKE_BYTES(&hardware->h_lladdr, ETHER_ADDR_LEN) &&
 	      /* LLDP frame */
 	      POKE_UINT16(ETHERTYPE_LLDP)))
 		goto toobig;
@@ -123,18 +123,22 @@ lldp_send(struct lldpd *global,
 		goto toobig;
 
 	/* System name */
-	if (!(
-	      POKE_START_LLDP_TLV(LLDP_TLV_SYSTEM_NAME) &&
-	      POKE_BYTES(chassis->c_name, strlen(chassis->c_name)) &&
-	      POKE_END_LLDP_TLV))
-		goto toobig;
+	if (chassis->c_name && *chassis->c_name != '\0') {
+		if (!(
+			    POKE_START_LLDP_TLV(LLDP_TLV_SYSTEM_NAME) &&
+			    POKE_BYTES(chassis->c_name, strlen(chassis->c_name)) &&
+			    POKE_END_LLDP_TLV))
+			goto toobig;
+	}
 
-	/* System description */
-	if (!(
-	      POKE_START_LLDP_TLV(LLDP_TLV_SYSTEM_DESCR) &&
-	      POKE_BYTES(chassis->c_descr, strlen(chassis->c_descr)) &&
-	      POKE_END_LLDP_TLV))
-		goto toobig;
+	/* System description (skip it if empty) */
+	if (chassis->c_descr && *chassis->c_descr != '\0') {
+		if (!(
+			    POKE_START_LLDP_TLV(LLDP_TLV_SYSTEM_DESCR) &&
+			    POKE_BYTES(chassis->c_descr, strlen(chassis->c_descr)) &&
+			    POKE_END_LLDP_TLV))
+			goto toobig;
+	}
 
 	/* System capabilities */
 	if (!(
@@ -178,11 +182,13 @@ lldp_send(struct lldpd *global,
 	}
 
 	/* Port description */
-	if (!(
-	      POKE_START_LLDP_TLV(LLDP_TLV_PORT_DESCR) &&
-	      POKE_BYTES(port->p_descr, strlen(port->p_descr)) &&
-	      POKE_END_LLDP_TLV))
-		goto toobig;
+	if (port->p_descr && *port->p_descr != '\0') {
+		if (!(
+			    POKE_START_LLDP_TLV(LLDP_TLV_PORT_DESCR) &&
+			    POKE_BYTES(port->p_descr, strlen(port->p_descr)) &&
+			    POKE_END_LLDP_TLV))
+			goto toobig;
+	}
 
 #ifdef ENABLE_DOT1
 	/* Port VLAN ID */
@@ -477,7 +483,7 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 	const char dot3[] = LLDP_TLV_ORG_DOT3;
 	const char med[] = LLDP_TLV_ORG_MED;
 	char orgid[3];
-	int length, gotend = 0;
+	int length, gotend = 0, ttl_received = 0;
 	int tlv_size, tlv_type, tlv_subtype;
 	u_int8_t *pos, *tlv;
 	char *b;
@@ -587,6 +593,7 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 		case LLDP_TLV_TTL:
 			CHECK_TLV_SIZE(2, "TTL");
 			chassis->c_ttl = PEEK_UINT16;
+			ttl_received = 1;
 			break;
 		case LLDP_TLV_PORT_DESCR:
 		case LLDP_TLV_SYSTEM_NAME:
@@ -805,8 +812,8 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 				hardware->h_rx_unrecognized_cnt++;
 #else
 				u_int32_t policy;
-				int loctype;
-				int power;
+				unsigned loctype;
+				unsigned power;
 
 				switch (tlv_subtype) {
 				case LLDP_TLV_MED_CAP:
@@ -920,8 +927,7 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 						port->p_med_power.devicetype =
 						    LLDP_MED_POW_TYPE_RESERVED;
 					}
-					if (((power & 0x0F) < 0) ||
-					    ((power & 0x0F) > LLDP_MED_POW_PRIO_LOW))
+					if ((power & 0x0F) > LLDP_MED_POW_PRIO_LOW)
 						port->p_med_power.priority =
 						    LLDP_MED_POW_PRIO_UNKNOWN;
 					else
@@ -973,10 +979,6 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 					case LLDP_TLV_MED_IV_ASSET:
 						chassis->c_med_asset = b;
 						break;
-					default:
-						log_warnx("lldp", "should not be there!");
-						free(b);
-						break;
 					}
 					port->p_med_cap_enabled |=
 					    LLDP_MED_CAP_IV;
@@ -1007,34 +1009,12 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 	/* Some random check */
 	if ((chassis->c_id == NULL) ||
 	    (port->p_id == NULL) ||
-	    (chassis->c_ttl == 0) ||
+	    (!ttl_received) ||
 	    (gotend == 0)) {
 		log_warnx("lldp", "some mandatory tlv are missing for frame received on %s",
 		    hardware->h_ifname);
 		goto malformed;
 	}
-#define NOTRECEIVED "Not received"
-	if (chassis->c_name == NULL) {
-		if ((chassis->c_name = (char *)calloc(1, strlen(NOTRECEIVED) + 1)) == NULL) {
-			log_warnx("lldp", "unable to allocate null chassis name");
-			goto malformed;
-		}
-		memcpy(chassis->c_name, NOTRECEIVED, strlen(NOTRECEIVED));
-	}
-	if (chassis->c_descr == NULL) {
-		if ((chassis->c_descr = (char *)calloc(1, strlen(NOTRECEIVED) + 1)) == NULL) {
-			log_warnx("lldp", "unable to allocate null chassis description");
-			goto malformed;
-		}
-		memcpy(chassis->c_descr, NOTRECEIVED, strlen(NOTRECEIVED));
-	}
-	if (port->p_descr == NULL) {
-		if ((port->p_descr = (char *)calloc(1, strlen(NOTRECEIVED) + 1)) == NULL) {
-			log_warnx("lldp", "unable to allocate null port description");
-			goto malformed;
-		}
-		memcpy(port->p_descr, NOTRECEIVED, strlen(NOTRECEIVED));
-	}
 	*newchassis = chassis;
 	*newport = port;
 	return 1;
