@@ -232,7 +232,7 @@ lldpd_display_neighbors(struct lldpd *cfg)
 		if (neighbors == 0)
 			priv_iface_description(hardware->h_ifname,
 			    "");
-		else if (neighbors == 1 && neighbor) {
+		else if (neighbors == 1 && neighbor && *neighbor != '\0') {
 			if (asprintf(&description, "%s",
 				neighbor) != -1) {
 				priv_iface_description(hardware->h_ifname, description);
@@ -263,7 +263,7 @@ lldpd_count_neighbors(struct lldpd *cfg)
 	neighbors--;
 	if (neighbors == 0)
 		setproctitle("no neighbor");
-	else if (neighbors == 1 && neighbor)
+	else if (neighbors == 1 && neighbor && *neighbor != '\0')
 		setproctitle("connected to %s", neighbor);
 	else
 		setproctitle("%d neighbor%s", neighbors,
@@ -301,6 +301,8 @@ lldpd_reset_timer(struct lldpd *cfg)
 		size_t output_len;
 		char save[offsetof(struct lldpd_port, p_id_subtype)];
 		memcpy(save, port, sizeof(save));
+		/* coverity[suspicious_sizeof]
+		   We intentionally partially memset port */
 		memset(port, 0, sizeof(save));
 		output_len = lldpd_port_serialize(port, (void**)&output);
 		memcpy(port, save, sizeof(save));
@@ -577,6 +579,8 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
 	   freed with lldpd_port_cleanup() and therefore, the refcount
 	   of the chassis that was attached to it is decreased.
 	*/
+	/* coverity[use_after_free]
+	   TAILQ_REMOVE does the right thing */
 	i = 0; TAILQ_FOREACH(aport, &hardware->h_rports, p_entries)
 		i++;
 	log_debug("decode", "%d neighbors for %s", i,
@@ -641,11 +645,11 @@ lldpd_get_lsb_release() {
 		return NULL;
 	}
 
-	if ((pid = fork()) < 0) {
+	pid = vfork();
+	switch (pid) {
+	case -1:
 		log_warn("localchassis", "unable to fork");
 		return NULL;
-	}
-	switch (pid) {
 	case 0:
 		/* Child, exec lsb_release */
 		close(pipefd[0]);
@@ -657,7 +661,7 @@ lldpd_get_lsb_release() {
 			if (pipefd[1] > 2) close(pipefd[1]);
 			execvp("lsb_release", command);
 		}
-		exit(127);
+		_exit(127);
 		break;
 	default:
 		/* Father, read the output from the children */
@@ -714,12 +718,12 @@ lldpd_get_os_release() {
 		return NULL;
 	}
 
-	while ((fgets(line, 1024, fp) != NULL)) {
+	while ((fgets(line, sizeof(line), fp) != NULL)) {
 		key = strtok(line, "=");
 		val = strtok(NULL, "=");
 
-		if (strncmp(key, "PRETTY_NAME", 1024) == 0) {
-			strncpy(release, val, 1024);
+		if (strncmp(key, "PRETTY_NAME", sizeof(line)) == 0) {
+			strlcpy(release, val, sizeof(line));
 			break;
 		}
 	}
@@ -983,8 +987,13 @@ lldpd_update_localchassis(struct lldpd *cfg)
 	/* Set system name and description */
 	if (uname(&un) < 0)
 		fatal("localchassis", "failed to get system information");
-	if ((hp = priv_gethostbyname()) == NULL)
-		fatal("localchassis", "failed to get system name");
+	if (cfg->g_config.c_hostname) {
+		log_debug("localchassis", "use overridden system name `%s`", cfg->g_config.c_hostname);
+		hp = cfg->g_config.c_hostname;
+	} else {
+		if ((hp = priv_gethostbyname()) == NULL)
+			fatal("localchassis", "failed to get system name");
+	}
 	free(LOCAL_CHASSIS(cfg)->c_name);
 	free(LOCAL_CHASSIS(cfg)->c_descr);
 	if ((LOCAL_CHASSIS(cfg)->c_name = strdup(hp)) == NULL)
@@ -1027,6 +1036,8 @@ lldpd_update_localchassis(struct lldpd *cfg)
 	else
 		LOCAL_CHASSIS(cfg)->c_med_sw = strdup("Unknown");
 #endif
+	if (LOCAL_CHASSIS(cfg)->c_cap_enabled == 0)
+		LOCAL_CHASSIS(cfg)->c_cap_enabled = LLDP_CAP_STATION;
 
 	/* Set chassis ID if needed. This is only done if chassis ID
 	   has not been set previously (with the MAC address of an
@@ -1104,9 +1115,15 @@ lldpd_exit(struct lldpd *cfg)
 static pid_t
 lldpd_configure(int debug, const char *path, const char *ctlname)
 {
-	pid_t lldpcli = fork();
+	pid_t lldpcli = vfork();
 	int devnull;
 
+	char sdebug[debug + 3];
+	memset(sdebug, 'd', debug + 3);
+	sdebug[debug + 2] = '\0';
+	sdebug[0] = '-'; sdebug[1] = 's';
+	log_debug("main", "invoke %s %s", path, sdebug);
+
 	switch (lldpcli) {
 	case -1:
 		log_warn("main", "unable to fork");
@@ -1114,27 +1131,20 @@ lldpd_configure(int debug, const char *path, const char *ctlname)
 	case 0:
 		/* Child, exec lldpcli */
 		if ((devnull = open("/dev/null", O_RDWR, 0)) != -1) {
-			char sdebug[debug + 3];
-			memset(sdebug, 'd', debug + 3);
-			sdebug[debug + 2] = '\0';
-			sdebug[0] = '-'; sdebug[1] = 's';
-
 			dup2(devnull,   STDIN_FILENO);
 			dup2(devnull,   STDOUT_FILENO);
 			if (devnull > 2) close(devnull);
 
-			log_debug("main", "invoke %s %s", path, sdebug);
-			if (execl(path, "lldpcli", sdebug,
-				"-u", ctlname,
-				"-c", SYSCONFDIR "/lldpd.conf",
-				"-c", SYSCONFDIR "/lldpd.d",
-				"resume",
-				NULL) == -1) {
-				log_warn("main", "unable to execute %s", path);
-				log_warnx("main", "configuration is incomplete, lldpd needs to be unpaused");
-			}
+			execl(path, "lldpcli", sdebug,
+			    "-u", ctlname,
+			    "-c", SYSCONFDIR "/lldpd.conf",
+			    "-c", SYSCONFDIR "/lldpd.d",
+			    "resume",
+			    NULL);
+			log_warn("main", "unable to execute %s", path);
+			log_warnx("main", "configuration is incomplete, lldpd needs to be unpaused");
 		}
-		exit(127);
+		_exit(127);
 		break;
 	default:
 		/* Father, don't do anything stupid */
@@ -1193,6 +1203,7 @@ lldpd_started_by_upstart()
 		return 0;
 	log_debug("main", "running with upstart, don't fork but stop");
 	raise(SIGSTOP);
+	unsetenv("UPSTART_JOB");
 	return 1;
 #else
 	return 0;
@@ -1234,6 +1245,7 @@ lldpd_started_by_systemd()
 		.msg_iov = &iov,
 		.msg_iovlen = 1
 	};
+	unsetenv("NOTIFY_SOCKET");
 	if (sendmsg(fd, &hdr, MSG_NOSIGNAL) < 0) {
 		log_warn("main", "unable to send notification to systemd");
 		close(fd);
@@ -1261,6 +1273,9 @@ lldpd_main(int argc, char *argv[], char *envp[])
 	char *mgmtp = NULL;
 	char *cidp = NULL;
 	char *interfaces = NULL;
+	/* We do not want more options here. Please add them in lldpcli instead
+	 * unless there is a very good reason. Most command-line options will
+	 * get deprecated at some point. */
 	char *popt, opts[] =
 		"H:vhkrdD:xX:m:u:4:6:I:C:p:M:P:S:iL:@                    ";
 	int i, found, advertise_version = 1;
@@ -1291,10 +1306,13 @@ lldpd_main(int argc, char *argv[], char *envp[])
 	/*
 	 * Get and parse command line options
 	 */
-	popt = strchr(opts, '@');
-	for (i=0; protos[i].mode != 0; i++)
-		*(popt++) = protos[i].arg;
-	*popt = '\0';
+	if ((popt = strchr(opts, '@')) != NULL) {
+		for (i=0;
+		     protos[i].mode != 0 && *popt != '\0';
+		     i++)
+			*(popt++) = protos[i].arg;
+		*popt = '\0';
+	}
 	while ((ch = getopt(argc, argv, opts)) != -1) {
 		switch (ch) {
 		case 'h':
@@ -1328,6 +1346,7 @@ lldpd_main(int argc, char *argv[], char *envp[])
 		case 'L':
 			if (strlen(optarg)) lldpcli = optarg;
 			else lldpcli = NULL;
+			break;
 		case 'k':
 			advertise_version = 0;
 			break;
@@ -1365,9 +1384,11 @@ lldpd_main(int argc, char *argv[], char *envp[])
 #endif
 			break;
                 case 'S':
+			free(descr_override);
                         descr_override = strdup(optarg);
                         break;
 		case 'P':
+			free(platform_override);
 			platform_override = strdup(optarg);
 			break;
 		case 'H':
@@ -1512,7 +1533,7 @@ lldpd_main(int argc, char *argv[], char *envp[])
 	cfg->g_snmp_agentx = agentx;
 #endif /* USE_SNMP */
 	cfg->g_config.c_bond_slave_src_mac_type = \
-	    LLDP_BOND_SLAVE_SRC_MAC_TYPE_FIXED;
+	    LLDP_BOND_SLAVE_SRC_MAC_TYPE_LOCALLY_ADMINISTERED;
 
 	/* Get ioctl socket */
 	log_debug("main", "get an ioctl socket");
@@ -1536,7 +1557,7 @@ lldpd_main(int argc, char *argv[], char *envp[])
 		calloc(1, sizeof(struct lldpd_chassis))) == NULL)
 		fatal("localchassis", NULL);
 	lchassis->c_cap_available = LLDP_CAP_BRIDGE | LLDP_CAP_WLAN |
-	    LLDP_CAP_ROUTER;
+	    LLDP_CAP_ROUTER | LLDP_CAP_STATION;
 	TAILQ_INIT(&lchassis->c_mgmt);
 #ifdef ENABLE_LLDPMED
 	if (lldpmed > 0) {
