@@ -17,7 +17,6 @@
 
 #ifndef _LLDPD_H
 #define _LLDPD_H
-#define _GNU_SOURCE 1
 
 #if HAVE_CONFIG_H
 #  include <config.h>
@@ -38,6 +37,13 @@
 #include <netinet/in.h>
 #include <sys/un.h>
 
+#if HAVE_VFORK_H
+# include <vfork.h>
+#endif
+#if HAVE_WORKING_FORK
+# define vfork fork
+#endif
+
 #include "lldp-tlv.h"
 #if defined (ENABLE_CDP) || defined (ENABLE_FDP)
 #  include "cdp.h"
@@ -67,7 +73,6 @@ struct event_base;
 #define LLDPD_TTL              LLDPD_TX_INTERVAL * LLDPD_TX_HOLD
 #define LLDPD_TX_MSGDELAY	1
 #define LLDPD_MAX_NEIGHBORS	4
-#define LLDPD_PID_FILE		"/var/run/lldpd.pid"
 #define LLDPD_FAST_TX_INTERVAL	1
 #define LLDPD_FAST_INIT	4
 
@@ -77,6 +82,8 @@ struct event_base;
 #define PROTO_DECODE_SIG struct lldpd *, char *, int, struct lldpd_hardware *, struct lldpd_chassis **, struct lldpd_port **
 #define PROTO_GUESS_SIG char *, int
 
+#define ALIGNED_CAST(TYPE, ATTR) ((TYPE) (void *) (ATTR))
+
 struct protocol {
 	int		 mode;		/* > 0 mode identifier (unique per protocol) */
 	int		 enabled;	/* Is this protocol enabled? */
@@ -199,8 +206,10 @@ void		 agent_init(struct lldpd *, char *);
 void		 agent_notify(struct lldpd_hardware *, int, struct lldpd_port *);
 #endif
 
+#ifdef ENABLE_PRIVSEP
 /* agent_priv.c */
 void		 agent_priv_register_domain(void);
+#endif
 
 /* client.c */
 int
@@ -212,6 +221,7 @@ client_handle_client(struct lldpd *cfg,
 
 /* priv.c */
 void	 priv_init(const char*, int, uid_t, gid_t);
+void	 priv_wait(void);
 void	 priv_ctl_cleanup(const char *ctlname);
 char   	*priv_gethostbyname(void);
 #ifdef HOST_OS_LINUX
@@ -240,17 +250,22 @@ enum priv_cmd {
 };
 
 /* priv-seccomp.c */
-#ifdef USE_SECCOMP
+#if defined USE_SECCOMP && defined ENABLE_PRIVSEP
 int priv_seccomp_init(int, int);
 #endif
 
 /* privsep_io.c */
-int	 may_read(void *, size_t);
-void	 must_read(void *, size_t);
-void	 must_write(const void *, size_t);
-void	 priv_remote(int);
-int	 receive_fd(void);
-void	 send_fd(int);
+enum priv_context {
+	PRIV_PRIVILEGED,
+	PRIV_UNPRIVILEGED
+};
+int	 may_read(enum priv_context, void *, size_t);
+void	 must_read(enum priv_context, void *, size_t);
+void	 must_write(enum priv_context, const void *, size_t);
+void	 priv_privileged_fd(int);
+void	 priv_unprivileged_fd(int);
+int	 receive_fd(enum priv_context);
+void	 send_fd(enum priv_context, int);
 
 /* interfaces-*.c */
 
@@ -362,7 +377,8 @@ void interfaces_helper_physical(struct lldpd *,
     struct interfaces_device_list *,
     struct lldpd_ops *,
     int(*init)(struct lldpd *, struct lldpd_hardware *));
-void interfaces_helper_port_name_desc(struct lldpd_hardware *,
+void interfaces_helper_port_name_desc(struct lldpd *,
+    struct lldpd_hardware *,
     struct interfaces_device *);
 void interfaces_helper_mgmt(struct lldpd *,
     struct interfaces_address_list *);
@@ -387,4 +403,7 @@ int netlink_subscribe_changes(void);
 int ifbpf_phys_init(struct lldpd *, struct lldpd_hardware *);
 #endif
 
+/* pattern.c */
+int pattern_match(char *, char *, int);
+
 #endif /* _LLDPD_H */
