@@ -268,6 +268,12 @@ static lldpctl_map_t bond_slave_src_mac_map[] = {
 	{ LLDP_BOND_SLAVE_SRC_MAC_TYPE_UNKNOWN, NULL},
 };
 
+static lldpctl_map_t lldp_portid_map[] = {
+	{ LLDP_PORTID_SUBTYPE_IFNAME,   "ifname"},
+	{ LLDP_PORTID_SUBTYPE_LLADDR,   "macaddress"},
+	{ LLDP_PORTID_SUBTYPE_UNKNOWN,  NULL},
+};
+
 static const char*
 map_lookup(lldpctl_map_t *list, int n)
 {
@@ -326,6 +332,8 @@ lldpctl_key_get_map(lldpctl_key_t key)
 #endif
 	case lldpctl_k_config_bond_slave_src_mac_type:
 		return bond_slave_src_mac_map;
+	case lldpctl_k_config_lldp_portid_type:
+		return lldp_portid_map;
 	default: return empty_map;
 	}
 }
@@ -367,9 +375,14 @@ _lldpctl_atom_get_str_config(lldpctl_atom_t *atom, lldpctl_key_t key)
 		res = c->config->c_description; break;
 	case lldpctl_k_config_platform:
 		res = c->config->c_platform; break;
+	case lldpctl_k_config_hostname:
+		res = c->config->c_hostname; break;
 	case lldpctl_k_config_bond_slave_src_mac_type:
 		return map_lookup(bond_slave_src_mac_map,
 				c->config->c_bond_slave_src_mac_type);
+	case lldpctl_k_config_lldp_portid_type:
+		return map_lookup(lldp_portid_map,
+		    c->config->c_lldp_portid_type);
 	default:
 		SET_ERROR(atom->conn, LLDPCTL_ERR_NOT_EXIST);
 		return NULL;
@@ -384,7 +397,7 @@ _lldpctl_atom_set_str_config(lldpctl_atom_t *atom, lldpctl_key_t key,
 	struct _lldpctl_atom_config_t *c =
 	    (struct _lldpctl_atom_config_t *)atom;
 	struct lldpd_config config = {};
-	char *iface_pattern = NULL;
+	char *iface_pattern = NULL, *mgmt_pattern = NULL;
 	char *description = NULL;
 	int rc, len;
 
@@ -400,6 +413,15 @@ _lldpctl_atom_set_str_config(lldpctl_atom_t *atom, lldpctl_key_t key,
 		free(c->config->c_iface_pattern);
 		c->config->c_iface_pattern = strdup(iface_pattern);
 		break;
+	case lldpctl_k_config_mgmt_pattern:
+		mgmt_pattern = _lldpctl_alloc_in_atom(atom, strlen(value) + 1);
+		if (!mgmt_pattern)
+			return NULL;
+		memcpy(mgmt_pattern, value, len);
+		config.c_mgmt_pattern = mgmt_pattern;
+		free(c->config->c_mgmt_pattern);
+		c->config->c_mgmt_pattern = strdup(mgmt_pattern);
+		break;
 	case lldpctl_k_config_description:
 		description = _lldpctl_alloc_in_atom(atom, strlen(value) + 1);
 		if (!description)
@@ -418,6 +440,15 @@ _lldpctl_atom_set_str_config(lldpctl_atom_t *atom, lldpctl_key_t key,
 		free(c->config->c_platform);
 		c->config->c_description = strdup(description);
 		break;
+	case lldpctl_k_config_hostname:
+		description = _lldpctl_alloc_in_atom(atom, strlen(value) + 1);
+		if (!description)
+			return NULL;
+		memcpy(description, value, len);
+		config.c_hostname = description;
+		free(c->config->c_hostname);
+		c->config->c_hostname = strdup(description);
+		break;
 	default:
 		SET_ERROR(atom->conn, LLDPCTL_ERR_NOT_EXIST);
 		return NULL;
@@ -501,6 +532,10 @@ _lldpctl_atom_set_int_config(lldpctl_atom_t *atom, lldpctl_key_t key,
 		config.c_bond_slave_src_mac_type = value;
 		c->config->c_bond_slave_src_mac_type = value;
 		break;
+	case lldpctl_k_config_lldp_portid_type:
+		config.c_lldp_portid_type = value;
+		c->config->c_lldp_portid_type = value;
+		break;
 	default:
 		SET_ERROR(atom->conn, LLDPCTL_ERR_NOT_EXIST);
 		return NULL;
@@ -513,7 +548,6 @@ _lldpctl_atom_set_int_config(lldpctl_atom_t *atom, lldpctl_key_t key,
 	    NULL, NULL);
 	if (rc == 0) return atom;
 	return NULL;
-		return atom;
 }
 
 static int
@@ -1920,12 +1954,13 @@ read_fixed_precision(lldpctl_atom_t *atom,
 		return NULL;
 	}
 
-	char *stored = _lldpctl_alloc_in_atom(atom, strlen(result) + 1);
+	size_t len = strlen(result) + 1;
+	char *stored = _lldpctl_alloc_in_atom(atom, len);
 	if (stored == NULL) {
 		free(result);
 		return NULL;
 	}
-	strlcpy(stored, result, strlen(result) + 1);
+	strlcpy(stored, result, len);
 	return stored;
 }
 
@@ -2292,6 +2327,7 @@ _lldpctl_atom_set_str_med_caelement(lldpctl_atom_t *atom, lldpctl_key_t key,
 {
 	struct _lldpctl_atom_med_caelement_t *el =
 	    (struct _lldpctl_atom_med_caelement_t *)atom;
+	size_t len;
 
 	/* Only local port can be modified */
 	if (el->parent->parent->hardware == NULL) {
@@ -2301,10 +2337,11 @@ _lldpctl_atom_set_str_med_caelement(lldpctl_atom_t *atom, lldpctl_key_t key,
 
 	switch (key) {
 	case lldpctl_k_med_civicaddress_value:
-		if (strlen(value) > 250) goto bad;
-		el->value = _lldpctl_alloc_in_atom(atom, strlen(value) + 1);
+		len = strlen(value) + 1;
+		if (len > 251) goto bad;
+		el->value = _lldpctl_alloc_in_atom(atom, len);
 		if (el->value == NULL) return NULL;
-		strlcpy((char*)el->value, value, strlen(value) + 1);
+		strlcpy((char*)el->value, value, len);
 		el->len = strlen(value);
 		return atom;
 	case lldpctl_k_med_civicaddress_type:
