@@ -589,6 +589,7 @@ typedef enum {
 	lldpctl_k_config_cid_pattern,	/**< `(S)` Interface pattern to choose the chassis ID */
 	lldpctl_k_config_description,	/**< `(S,WO)` Chassis description overridden */
 	lldpctl_k_config_platform,	/**< `(S,WO)` Platform description overridden (CDP) */
+	lldpctl_k_config_hostname,	/**< `(S,WO)` System name overridden */
 	lldpctl_k_config_advertise_version, /**< `(I)` Advertise version */
 	lldpctl_k_config_lldpmed_noinventory, /**< `(I)` Disable LLDP-MED inventory */
 	lldpctl_k_config_paused,	      /**< `(I)` lldpd is paused */
@@ -709,6 +710,7 @@ typedef enum {
 	lldpctl_k_delete_cnt,	/**< `(I)` delete cnt. Only works for a local port. */
 	lldpctl_k_config_tx_hold, /**< `(I,WO)` Transmit hold interval. */
 	lldpctl_k_config_bond_slave_src_mac_type, /**< `(I,WO)` bond slave src mac type. */
+	lldpctl_k_config_lldp_portid_type, /**< `(I,WO)` LLDP PortID TLV Subtype */
 } lldpctl_key_t;
 
 /**
