@@ -323,6 +323,7 @@ struct lldpd_config {
 
 	char *c_platform;	/* Override platform description (for CDP) */
 	char *c_description;	/* Override chassis description */
+	char *c_hostname;	/* Override system name */
 	int c_advertise_version; /* Should the precise version be advertised? */
 	int c_set_ifdescr;	 /* Set interface description */
 
@@ -335,11 +336,13 @@ struct lldpd_config {
 	int c_tx_hold;		/* Transmit hold */
 	int c_bond_slave_src_mac_type; /* Src mac type in lldp frames over bond
 					  slaves */
+	int c_lldp_portid_type; /* The PortID type */
 };
 MARSHAL_BEGIN(lldpd_config)
 MARSHAL_STR(lldpd_config, c_mgmt_pattern)
 MARSHAL_STR(lldpd_config, c_cid_pattern)
 MARSHAL_STR(lldpd_config, c_iface_pattern)
+MARSHAL_STR(lldpd_config, c_hostname)
 MARSHAL_STR(lldpd_config, c_platform)
 MARSHAL_STR(lldpd_config, c_description)
 MARSHAL_END(lldpd_config);
